"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.withLogger = exports.createCodemirror = void 0;
const autocomplete_1 = require("@codemirror/autocomplete");
const commands_1 = require("@codemirror/commands");
const language_1 = require("@codemirror/language");
const view_1 = require("@codemirror/view");
const action_names_1 = require("../../bundle/config/action-names");
const utils_1 = require("../../extensions/behavior/Clipboard/utils");
const logger_1 = require("../../logger");
const shortcuts_1 = require("../../shortcuts");
const commands_2 = require("../commands");
const files_upload_facet_1 = require("./files-upload-facet");
const gravity_1 = require("./gravity");
const pairing_chars_1 = require("./pairing-chars");
const react_facet_1 = require("./react-facet");
const plugin_1 = require("./search-plugin/plugin");
const yfm_1 = require("./yfm");
function createCodemirror(params) {
    const { doc, reactRenderer, onCancel, onScroll, onSubmit, onChange, onDocChange, disabledExtensions = {}, keymaps = [], receiver, yfmLangOptions, extensions: extraExtensions, placeholder: placeholderContent, autocompletion: autocompletionConfig, parseInsertedUrlAsImage, } = params;
    const extensions = [gravity_1.gravityTheme, (0, view_1.placeholder)(placeholderContent)];
    if (!disabledExtensions.history) {
        extensions.push((0, commands_1.history)());
    }
    extensions.push((0, language_1.syntaxHighlighting)(gravity_1.gravityHighlightStyle), view_1.keymap.of([
        { key: shortcuts_1.formatter.toCM(shortcuts_1.Action.Bold), run: withLogger(action_names_1.ActionName.bold, commands_2.toggleBold) },
        { key: shortcuts_1.formatter.toCM(shortcuts_1.Action.Italic), run: withLogger(action_names_1.ActionName.italic, commands_2.toggleItalic) },
        { key: shortcuts_1.formatter.toCM(shortcuts_1.Action.Strike), run: withLogger(action_names_1.ActionName.strike, commands_2.toggleStrikethrough) },
        { key: shortcuts_1.formatter.toCM(shortcuts_1.Action.Underline), run: withLogger(action_names_1.ActionName.underline, commands_2.toggleUnderline) },
        { key: shortcuts_1.formatter.toCM(shortcuts_1.Action.Link), run: withLogger(action_names_1.ActionName.link, commands_2.insertLink) },
        { key: shortcuts_1.formatter.toCM(shortcuts_1.Action.Heading1), run: withLogger(action_names_1.ActionName.heading1, commands_2.toH1) },
        { key: shortcuts_1.formatter.toCM(shortcuts_1.Action.Heading2), run: withLogger(action_names_1.ActionName.heading2, commands_2.toH2) },
        { key: shortcuts_1.formatter.toCM(shortcuts_1.Action.Heading3), run: withLogger(action_names_1.ActionName.heading3, commands_2.toH3) },
        { key: shortcuts_1.formatter.toCM(shortcuts_1.Action.Heading4), run: withLogger(action_names_1.ActionName.heading4, commands_2.toH4) },
        { key: shortcuts_1.formatter.toCM(shortcuts_1.Action.Heading5), run: withLogger(action_names_1.ActionName.heading5, commands_2.toH5) },
        { key: shortcuts_1.formatter.toCM(shortcuts_1.Action.Heading6), run: withLogger(action_names_1.ActionName.heading6, commands_2.toH6) },
        { key: shortcuts_1.formatter.toCM(shortcuts_1.Action.Code), run: withLogger(action_names_1.ActionName.code_inline, commands_2.wrapToInlineCode) },
        { key: shortcuts_1.formatter.toCM(shortcuts_1.Action.CodeBlock), run: withLogger(action_names_1.ActionName.code_block, commands_2.wrapToCodeBlock) },
        { key: shortcuts_1.formatter.toCM(shortcuts_1.Action.Cut), run: withLogger(action_names_1.ActionName.yfm_cut, commands_2.wrapToYfmCut) },
        { key: shortcuts_1.formatter.toCM(shortcuts_1.Action.Note), run: withLogger(action_names_1.ActionName.yfm_note, commands_2.wrapToYfmNote) },
        {
            key: shortcuts_1.formatter.toCM(shortcuts_1.Action.Cancel),
            preventDefault: true,
            run: () => {
                onCancel();
                return true;
            },
        },
        {
            key: shortcuts_1.formatter.toCM(shortcuts_1.Action.Submit),
            preventDefault: true,
            run: () => {
                onSubmit();
                return true;
            },
        },
        { key: 'Tab', preventDefault: true, run: commands_1.insertTab },
        {
            key: 'Enter',
            shift: commands_1.insertNewlineKeepIndent,
        },
        commands_1.indentWithTab,
        ...commands_1.defaultKeymap,
        ...(disabledExtensions.history ? [] : commands_1.historyKeymap),
        ...keymaps,
    ]), (0, autocomplete_1.autocompletion)(autocompletionConfig), (0, yfm_1.yfmLang)(yfmLangOptions), react_facet_1.ReactRendererFacet.of(reactRenderer), pairing_chars_1.PairingCharactersExtension, view_1.EditorView.lineWrapping, view_1.EditorView.contentAttributes.of({ spellcheck: 'true' }), view_1.EditorView.domEventHandlers({
        scroll(event) {
            onScroll(event);
        },
        paste(event, editor) {
            var _a;
            if (event.clipboardData && parseInsertedUrlAsImage) {
                const { imageUrl, title } = parseInsertedUrlAsImage((_a = event.clipboardData.getData(utils_1.DataTransferType.Text)) !== null && _a !== void 0 ? _a : '') || {};
                if (!imageUrl) {
                    return;
                }
                event.preventDefault();
                (0, commands_2.insertImages)([
                    {
                        url: imageUrl,
                        alt: title,
                        title,
                    },
                ])(editor);
            }
        },
    }), (0, plugin_1.SearchPanelPlugin)({
        anchorSelector: '.g-md-search-anchor',
        receiver,
    }));
    if (params.uploadHandler) {
        extensions.push(files_upload_facet_1.FileUploadHandlerFacet.of({
            fn: params.uploadHandler,
            imageWithDimensions: params.needImageDimensions,
            enableNewImageSizeCalculation: params.enableNewImageSizeCalculation,
        }));
    }
    if (extraExtensions) {
        extensions.push(...extraExtensions);
    }
    return new view_1.EditorView({
        doc,
        extensions,
        dispatchTransactions: (trs, view) => {
            view.update(trs);
            onChange();
            if (trs.some((tr) => tr.docChanged)) {
                onDocChange();
            }
        },
    });
}
exports.createCodemirror = createCodemirror;
function withLogger(action, command) {
    return (...args) => {
        logger_1.logger.action({ mode: 'markup', source: 'keymap', action });
        return command(...args);
    };
}
exports.withLogger = withLogger;
