import React, { useEffect, useMemo, useRef, useState } from 'react';
import { ErrorBoundary } from 'react-error-boundary';
import { useEnsuredForwardedRef, useKey, useUpdate } from 'react-use';
import { cn } from '../classname';
import { i18n } from '../i18n/bundle';
import { logger } from '../logger';
import { ToasterContext, useBooleanState, useSticky } from '../react-utils';
import { isMac } from '../utils';
import { HorizontalDrag } from './HorizontalDrag';
import { MarkupEditorView } from './MarkupEditorView';
import { SplitModeView } from './SplitModeView';
import { WysiwygEditorView } from './WysiwygEditorView';
import { mHiddenDataByPreset, mToolbarConfigByPreset, wHiddenDataByPreset, wToolbarConfigByPreset, } from './config';
import { useMarkdownEditorContext } from './context';
import { EditorSettings } from './settings';
import { stickyCn } from './sticky';
import '../styles/styles.css';
import './MarkdownEditorView.css'; // eslint-disable-line import/order
export const cnEditorComponent = cn('editor-component');
const b = cnEditorComponent;
export const MarkdownEditorView = React.forwardRef((props, ref) => {
    var _a, _b;
    const divRef = useEnsuredForwardedRef(ref);
    const [isMounted, setIsMounted] = useState(false);
    useEffect(() => {
        setIsMounted(true);
    }, []);
    const [showPreview, , unsetShowPreview, toggleShowPreview] = useBooleanState(false);
    const context = useMarkdownEditorContext();
    const editor = ((_a = props.editor) !== null && _a !== void 0 ? _a : context);
    if (!editor)
        throw new Error('[MarkdownEditorView]: an instance of the editor must be passed through the props or context');
    const { autofocus, className, settingsVisible = true, markupToolbarConfig = mToolbarConfigByPreset[editor.preset], wysiwygToolbarConfig = wToolbarConfigByPreset[editor.preset], markupHiddenActionsConfig = mHiddenDataByPreset[editor.preset], wysiwygHiddenActionsConfig = wHiddenDataByPreset[editor.preset], toaster, stickyToolbar, enableSubmitInPreview = true, hidePreviewAfterSubmit = false, } = props;
    const rerender = useUpdate();
    React.useLayoutEffect(() => {
        editor.on('rerender', rerender);
        return () => {
            editor.off('rerender', rerender);
        };
    }, [editor, rerender]);
    const onModeChange = React.useCallback((type) => {
        editor.changeEditorMode({ mode: type, reason: 'settings' });
        unsetShowPreview();
    }, [editor, unsetShowPreview]);
    const onToolbarVisibilityChange = React.useCallback((visible) => {
        editor.changeToolbarVisibility({ visible });
    }, [editor]);
    const onSplitModeChange = React.useCallback((splitModeEnabled) => {
        unsetShowPreview();
        editor.changeSplitModeEnabled({ splitModeEnabled });
    }, [editor, unsetShowPreview]);
    const onShowPreviewChange = React.useCallback((showPreviewValue) => {
        editor.changeSplitModeEnabled({ splitModeEnabled: false });
        if (showPreviewValue !== showPreview)
            toggleShowPreview();
    }, [editor, showPreview, toggleShowPreview]);
    const editorMode = editor.currentMode;
    const markupSplitMode = editor.splitModeEnabled && editor.splitMode && editorMode === 'markup';
    const canRenderPreview = Boolean(editor.renderPreview && editorMode === 'markup' && !editor.splitModeEnabled);
    useKey((e) => canRenderPreview && isPreviewKeyDown(e), (e) => {
        e.preventDefault();
        onShowPreviewChange(!showPreview);
    }, { event: 'keydown' }, [showPreview, editorMode, onShowPreviewChange, canRenderPreview]);
    const editorWrapperRef = useRef(null);
    const splitModeViewWrapperRef = useRef(null);
    useEffect(() => {
        if (showPreview) {
            divRef.current.focus();
        }
    }, [divRef, showPreview]);
    useKey((e) => enableSubmitInPreview &&
        showPreview &&
        isWrapperFocused(divRef) &&
        isSubmitKeyDown(e), () => {
        editor.emit('submit', null);
        if (hidePreviewAfterSubmit) {
            onShowPreviewChange(false);
        }
    }, { event: 'keydown' }, [hidePreviewAfterSubmit, enableSubmitInPreview, showPreview, showPreview]);
    const settings = useMemo(() => (React.createElement(Settings, { mode: editorMode, settingsVisible: settingsVisible, onModeChange: onModeChange, toolbarVisibility: editor.toolbarVisible && !showPreview, onToolbarVisibilityChange: onToolbarVisibilityChange, onSplitModeChange: onSplitModeChange, splitModeEnabled: editor.splitModeEnabled, splitMode: editor.splitMode, stickyToolbar: stickyToolbar, onShowPreviewChange: onShowPreviewChange, showPreview: showPreview, renderPreviewButton: canRenderPreview })), [
        editorMode,
        settingsVisible,
        editor.toolbarVisible,
        editor.splitModeEnabled,
        editor.splitMode,
        onModeChange,
        showPreview,
        onToolbarVisibilityChange,
        onSplitModeChange,
        stickyToolbar,
        onShowPreviewChange,
        canRenderPreview,
    ]);
    return (React.createElement(ErrorBoundary, { onError: (e) => {
            logger.error(e);
        }, fallbackRender: ({ error, resetErrorBoundary }) => {
            toaster.add({
                theme: 'danger',
                name: 'g-md-editor-error',
                title: i18n('error-title'),
                content: error.message,
            });
            setTimeout(() => {
                resetErrorBoundary();
                editor.changeEditorMode({
                    mode: 'markup',
                    reason: 'error-boundary',
                    emit: false,
                });
            });
            return null;
        } },
        React.createElement(ToasterContext.Provider, { value: toaster },
            React.createElement("div", { ref: divRef, className: b({
                    settings: settingsVisible,
                    split: markupSplitMode && editor.splitMode,
                }, [className]), role: "button", tabIndex: 0 },
                React.createElement("div", { className: b('editor-wrapper'), ref: editorWrapperRef }, showPreview ? (React.createElement(React.Fragment, null,
                    React.createElement("div", { className: b('preview-wrapper') }, (_b = editor.renderPreview) === null || _b === void 0 ? void 0 : _b.call(editor, {
                        getValue: editor.getValue,
                        mode: 'preview',
                        md: editor.mdOptions,
                    })),
                    settings)) : (React.createElement(React.Fragment, null,
                    editorMode === 'wysiwyg' && (React.createElement(WysiwygEditorView, { editor: editor, autofocus: autofocus, settingsVisible: settingsVisible, toolbarConfig: wysiwygToolbarConfig, toolbarVisible: editor.toolbarVisible, hiddenActionsConfig: wysiwygHiddenActionsConfig, className: b('editor', { mode: editorMode }), toolbarClassName: b('toolbar'), stickyToolbar: stickyToolbar }, editor.toolbarVisible && settingsVisible && settings)),
                    editorMode === 'markup' && (React.createElement(MarkupEditorView, { editor: editor, autofocus: autofocus, settingsVisible: settingsVisible, toolbarConfig: markupToolbarConfig, toolbarVisible: editor.toolbarVisible, splitMode: editor.splitMode, splitModeEnabled: editor.splitModeEnabled, hiddenActionsConfig: markupHiddenActionsConfig, className: b('editor', { mode: editorMode }), toolbarClassName: b('toolbar'), stickyToolbar: stickyToolbar }, editor.toolbarVisible && settings)),
                    !editor.toolbarVisible && settings))),
                markupSplitMode && (React.createElement(React.Fragment, null,
                    editor.splitMode === 'horizontal' ? (React.createElement(HorizontalDrag, { editor: editor, isMounted: isMounted, leftElRef: editorWrapperRef, rightElRef: splitModeViewWrapperRef, wrapperRef: divRef })) : (React.createElement("div", { className: b('resizer') })),
                    React.createElement(SplitModeView, { editor: editor, ref: splitModeViewWrapperRef })))))));
});
MarkdownEditorView.displayName = 'MarkdownEditorView';
const MarkupSearchAnchor = ({ mode }) => (React.createElement(React.Fragment, null, mode === 'markup' && React.createElement("div", { className: "g-md-search-anchor" })));
function Settings(props) {
    const wrapperRef = useRef(null);
    const isSticky = useSticky(wrapperRef) && props.toolbarVisibility && props.stickyToolbar;
    return (React.createElement(React.Fragment, null, (props.renderPreviewButton || props.settingsVisible) && (React.createElement("div", { className: b('settings-wrapper') },
        React.createElement("div", { ref: wrapperRef, className: stickyCn.settings({
                withToolbar: props.toolbarVisibility,
                stickyActive: isSticky,
            }) },
            React.createElement(EditorSettings, Object.assign({}, props)),
            React.createElement(MarkupSearchAnchor, Object.assign({}, props)))))));
}
function isPreviewKeyDown(e) {
    const modKey = isMac() ? e.metaKey : e.ctrlKey;
    return modKey && e.shiftKey && e.code === 'KeyP';
}
function isWrapperFocused(divRef) {
    return document.activeElement === divRef.current;
}
function isSubmitKeyDown(e) {
    const modKey = isMac() ? e.metaKey : e.ctrlKey;
    return modKey && e.code === 'Enter';
}
